! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem 06d (2-dim.)
!        y(x) = [(1/2x)^4*exp(4-2x);(1/2x)^4*exp(4-2*x)*(4-2x)] (EW 0,0)
! ----------------------------------------------------------------------
MODULE fun
!   USE bessel
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   LOGICAL, PARAMETER :: linear = .true. 
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar dummy

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

!   CALL setupneu()      ! Neumannfunktionen vorbereiten
   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 0.0_lrk, 0.0_lrk /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y), mit Maple ueberprueft
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0_lrk) THEN
      fcn = (/ 0.0_lrk, 0.0_lrk /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(2)/x, x*4.0_lrk*y(1) + (x/2.0_lrk)**3/2.0_lrk* &
               exp(4.0_lrk-2.0_lrk*x)*(16.0_lrk-18.0_lrk*x) /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x), mit Maple eingesetzt.
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ (x/2.0_lrk)**4*exp(4.0_lrk-2.0_lrk*x), &
             (x/2.0_lrk)**4*exp(4.0_lrk-2.0_lrk*x)*(4.0_lrk-2.0_lrk*x) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Loesung abhaengig von xstart und Startwerten dummy
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   truelocal = yx0 - true(xstart,yx0) + true(x,yx0)
   RETURN
END FUNCTION truelocal

!FUNCTION truelocal(x, xstart, yx0)
!!        echte Loesung abhaengig von xstart und Startwerten, Loesungsschar,
!!        fuer P06x Bsp. wo I_0(alpha) als allg. Lsg herauskommt.
!   IMPLICIT NONE
!   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
!   REAL (KIND = lrk) :: c1, c2, v(neqn), a, k0, k1, i0, i1
!!        x ... aktueller Punkt
!!        xstart ... Beginn der Loesung
!!        yx0 ... Vektor von y(xstart)
!   REAL (KIND = lrk) :: truelocal(SIZE(yx0))
!
!   a = 2.0_lrk                  ! 8, 128, 32, 2
!   v = yx0 - true(xstart,yx0)
!   if (xstart == 0.0_lrk) THEN
!      c1 = v(1)/besselI0(a*xstart)
!      c2 = 0._lrk
!      a = a*x
!      truelocal = (/ c1*besselI0(a), c1*a*besselI1(a) /) + true(x,yx0)
!   ELSE
!      c2 = a*xstart
!      i0 = besselI0(c2)
!      i1 = besselI1(c2)
!      k0 = besselK0(c2)
!      k1 = besselK1(c2)
!      c1 = (v(1)*c2*k1+v(2)*k0)/(i0*k1+i1*k0)/c2
!      c2 = (v(1)-c1*i0)/k0
!      a = a*x
!      truelocal = (/ c1*besselI0(a) + c2*besselK0(a), &
!                    (c1*besselI1(a) - c2*besselK1(a))*a /) + true(x,yx0)
!   END IF
!   RETURN
!END FUNCTION truelocal

! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0_lrk, 1.0_lrk/x /)
      dfcn(2,:) = (/ 4.0_lrk*x, 0.0_lrk /)
   RETURN
END FUNCTION dfcn


! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!        Eigenwerte 1. frei (EW = 0), 2. muss 0 sein (EW <= 0)
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_lrk
   xend = 1.0_lrk
   u0 = true(xstart,u0) 
   v0 = (/ .false., .true. /) ! 2. Wert von negativem EW
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
