! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem 22a (2-dim.)
!        y(x) = [x**2*exp(k*x);...] (Eigenwerte -1, -2)
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   INTEGER, PARAMETER :: k = 2
!  p22b k=5, k=10
   LOGICAL, PARAMETER :: linear = .true.
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)
   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 0.0_lrk, 0.0_lrk /)
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   IF (x == 0.0_lrk) THEN
      fcn = (/ 0.0_lrk, 0.0_lrk /)
!        bei Null nicht auswertbar. Ableitung mittels Taylor berechnet.
   ELSE
      fcn = (/ y(2)/x, -2.0_lrk*y(1)/x -3.0_lrk*y(2)/x + &
               exp(x*k) * ((k*x+8.0_lrk)*k*x+12.0_lrk)*x /)
   END IF
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ x**2*exp(x*k), x**2*exp(x*k)*(2.0_lrk+x*k) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Loesung abhaengig von xstart und Startwerten
!        fuer lineare, konst. Koeff, l1=-1,l2=-2.
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = true(xstart,yx0)          ! t enthaelt die partikulaere Lsg. bei xstart
   c2 = xstart**2*(t(1)+t(2)-yx0(1)-yx0(2))
   IF (c2 == 0.0_lrk) THEN
        c1 = xstart*(yx0(1)-t(1))
   ELSE
        c1 = xstart*(yx0(1)-t(1))-c2/xstart
   END IF

   truelocal = (/ c1/x + c2/x/x, -c1/x -2.0_lrk*c2/x/x /) + true(x,yx0)
   RETURN
END FUNCTION truelocal
 
! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

      dfcn(1,:) = (/ 0.0_lrk, 1.0_lrk/x /)
      dfcn(2,:) = (/ -2.0_lrk/x, -3.0_lrk/x /)
   RETURN
END FUNCTION dfcn

END MODULE fun
