! ----------------------------------------------------------------------
!        FWF-Projekt                                    Dr. Peter KOFLER
!        Problem regulaer 07 (2-dim.)
!        y(x) = [10+5*x+2*exp((5+2*sqrt(5))*x), ()']
! ----------------------------------------------------------------------
MODULE fun
   IMPLICIT NONE
   INTEGER, PARAMETER :: lrk = KIND(1.0D0)                  
   INTEGER, PARAMETER :: neqn = 2
   REAL(KIND = lrk) :: f1s, f2s
   LOGICAL, PARAMETER :: linear = .true.
   PRIVATE
   PUBLIC :: lrk, neqn, linear, initial, bound, fcn, dfcn, true, truelocal
!        INTEGER neqn ............. Dimension des Systems
!        LOGICAL linear ........... Beispiel ist linear
!        SUBROUTINE initial ....... Anfangsbedingungen
!        REAL FUNTION fcn ......... f(x,y) (rechte Seite)
!        REAL FUNTION dfcn ........ df/dy(x,y) (Jakobiu Matrix)
!        REAL FUNTION true ........ y(x) (Loesung)
!        REAL FUNTION truelocal ... Loesungsschar

   CONTAINS
! ----------------------------------------------------------------------
SUBROUTINE initial(xstart, xend, yx0)
!        Angabe der Anfangsbedingungen und der Start- und Endpunkte
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, yx0(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        yx0 ... Vektor von y(xstart)

   xstart = 0.0_lrk
   xend = 1.0_lrk
   yx0 = (/ 12.0_lrk, 15.0_lrk + 4.0_lrk * SQRT(5.0_lrk) /)
   f1s = (5.0_lrk+2.0_lrk*SQRT(5.0_lrk))
   f2s = (5.0_lrk-2.0_lrk*SQRT(5.0_lrk))
   RETURN
END SUBROUTINE initial

! ----------------------------------------------------------------------
FUNCTION fcn(x, y)
!        Ableitung von y an x, also f(x,y)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: fcn(SIZE(y))

   fcn = (/ y(2), 10.0_lrk*y(2) - 5.0_lrk*y(1) + 25.0_lrk*x /)
   RETURN
END FUNCTION fcn

! ----------------------------------------------------------------------
FUNCTION true(x,y)
!        echte Loesung y(x)
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
   REAL (KIND = lrk) :: true(SIZE(y))

   true = (/ 10.0_lrk + 5.0_lrk*x + 2.0_lrk * EXP(f1s*x), &
             5.0_lrk + 2.0_lrk*f1s * EXP(f1s*x) /)
   RETURN
END FUNCTION true

! ----------------------------------------------------------------------
FUNCTION truelocal(x, xstart, yx0)
!        echte Loesung abhaengig von xstart und Startwerten
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, xstart, yx0(:)
   REAL (KIND = lrk) :: c1, c2, t(SIZE(yx0))
!        x ... aktueller Punkt
!        xstart ... Beginn der Loesung
!        yx0 ... Vektor von y(xstart)
   REAL (KIND = lrk) :: truelocal(SIZE(yx0))

   t = yx0 - (/ 10.0_lrk+5.0_lrk*xstart, 5.0_lrk /)
!         t enthaelt hom. Lsg. bei xstart, aus denen jetzt c1 und c2
!         berechnet werden.

   c1 = ( -t(1)*f2s + t(2) ) /4.0_lrk/SQRT(5.0_lrk)/EXP( f1s*xstart )
   c2 = ( t(1)-c1 * EXP(f1s*xstart) ) / EXP(f2s*xstart)

   truelocal = (/ c1* EXP(f1s*x) + c2* EXP(f2s*x) + 10.0_lrk+5.0_lrk*x, &
                  c1*f1s* EXP(f1s*x) + c2*f2s* EXP(f2s*x) + 5.0_lrk /)
   RETURN
END FUNCTION truelocal
 
! ----------------------------------------------------------------------
FUNCTION dfcn(x, y)
!        Jacobimatrix, also fcn nach y an x, also f_y(x,y) dummy
   IMPLICIT NONE
   REAL (KIND = lrk), INTENT(in) :: x, y(:)
!        x ... aktueller Punkt
!        y ... Wert von y(x)
   REAL (KIND = lrk) :: dfcn(SIZE(y),SIZE(y))

   dfcn(1,:) = (/ x-x, 1.0_lrk /)
   dfcn(2,:) = (/ -5.0_lrk, 10.0_lrk /)

   RETURN
END FUNCTION dfcn


! ----------------------------------------------------------------------
SUBROUTINE bound(xstart, xend, u0, u1, v0, v1)
!        Angabe der linearen Randbedingungen, des Start- und Endpunktes
!        Parameter frei nach NAG: D02HAF
!        Eigenwerte 1. frei (EW = 0), 2. muss 0 sein (EW <= 0)
   IMPLICIT NONE
   REAL(KIND = lrk), INTENT(out) :: xstart, xend, u0(:), u1(:)
!        xstart ... Beginn der Integration. meist 0
!        xend ... Endpunkt der Integration
!        u0, u1 ... Vektor von y(xstart) und y(xend), wobei unbekannte
!            ausgelassen werden.
   LOGICAL, INTENT(out) :: v0(:), v1(:)
!        v0, v1 ... v0(i) == .true. wenn Wert exakt ist, sonst .false.

   xstart = 0.0_lrk
   xend = 1.0_lrk
   u0 = true(xstart,u0) 
   v0 = (/ .false., .true. /) 
! v0 = .not. v0
   u1 = true(xend,u1)
   v1 = .not. v0
   RETURN
END SUBROUTINE bound

END MODULE fun
